#ifndef ARMARX_SKILLS_GUI_SKILLTREEWIDGET_H
#define ARMARX_SKILLS_GUI_SKILLTREEWIDGET_H

#include <QTreeWidget>

#include <RobotAPI/libraries/skills/core/SkillID.h>

#include "../memory/MemoryCommunicatorBase.h"

namespace armarx::skills::gui
{

    class SkillTreeWidget : public QTreeWidget, public MemoryCommunicatorBase
    {
        Q_OBJECT
    public:
        SkillTreeWidget(std::shared_ptr<SkillManagerWrapper> _memory, QWidget* parent = nullptr) :
            QTreeWidget(parent), MemoryCommunicatorBase(_memory)
        {
            setupUi();
        }

        struct SelectedSkill
        {
            skills::SkillID skillId;

            // make default constructable
            SelectedSkill() :
                skillId({.providerId = std::nullopt, .skillName = skills::SkillID::UNKNOWN})
            {
            }
        };

        SelectedSkill& getSelectedSkill();

    signals:
        /**
         * @brief Notify the skill description widget of an updated selection.
         * @param skillId
         */
        void updateSkillDetails(skills::SkillID& skillId);

    public slots:
        void disconnectGui();
        void updateSkills();

    private slots:
        void skillSelectionChanged(QTreeWidgetItem* current, QTreeWidgetItem* previous);


    private:
        SelectedSkill selectedSkill;

        void setupUi();
        void connectSignals();
    };

} // namespace armarx::skills::gui

#endif // ARMARX_SKILLS_GUI_SKILLTREEWIDGET_H
